@@ -26,8 +26,13 @@
 open Interface
 open Curses
 open Remind
+open Utility
 
 
+(* sort timed lineinfo entries by starting timestamp *)
+let sort_lineinfo (_, _, _, _, a) (_, _, _, _, b) =
+   ~- (Pervasives.compare a b)
+
 
 (* Word-wrap a string--split the string at whitespace boundaries
  * to form a list of strings, each of which has length less than
@@ -130,9 +135,10 @@ let draw_help (iface : interface_state_t) =
             build_help_line tail s
    in
    let help_string = 
-      build_help_line [(Rcfile.NewTimed, "new timed"); (Rcfile.NewUntimed, "new untimed");
-                       (Rcfile.Edit, "edit"); (Rcfile.Home, "home"); (Rcfile.Zoom, "zoom"); 
-                       (Rcfile.BeginSearch, "search"); (Rcfile.Quit, "quit")] ""
+      build_help_line [(Rcfile.ViewKeybindings, "help"); (Rcfile.NewTimed, "new timed"); 
+                       (Rcfile.NewUntimed, "new untimed"); (Rcfile.Edit, "edit"); 
+                       (Rcfile.Home, "home"); (Rcfile.Zoom, "zoom"); (Rcfile.BeginSearch, "search"); 
+                       (Rcfile.Quit, "quit")] ""
    in
    trunc_mvwaddstr iface.scr.help_win 0 0 iface.scr.hw_cols help_string;
    assert (wnoutrefresh iface.scr.help_win)
@@ -285,11 +291,14 @@ let draw_timed_window iface reminders top lines =
       else
          twentyfour_hour_string
    in
-   let desc_string_of_tm =
+   (* this ensures that if there are multiple reminder descriptions
+    * displayed simultaneously, the dashes between start and end times
+    * will line up properly *)
+   let (desc_string_of_tm1, desc_string_of_tm2) =
       if !Rcfile.description_12_hour then
-         twelve_hour_string
+         (twelve_hour_string_pad, twelve_hour_string)
       else
-         twentyfour_hour_string
+         (twentyfour_hour_string, twentyfour_hour_string)
    in
    (* draw in the blank timeslots *)
    Rcfile.color_on iface.scr.timed_win Rcfile.Timed_default;
@@ -302,32 +311,43 @@ let draw_timed_window iface reminders top lines =
       let ts = timestamp_of_line iface i in
       let tm = Unix.localtime ts in
       let ts_str = string_of_tm tm in
-      let s = Str.string_before (ts_str ^ blank) (iface.scr.tw_cols - 2) in
+      let curr_ts = Unix.time () in
+      (* the current time is highlighted *)
+      if curr_ts >= ts && curr_ts < (timestamp_of_line iface (succ i)) then begin 
+         Rcfile.color_on iface.scr.timed_win Rcfile.Timed_current;
+         wattron iface.scr.timed_win A.bold
+      end else
+         ();
       if tm.Unix.tm_hour = before_midnight.Unix.tm_hour &&
          tm.Unix.tm_min  = before_midnight.Unix.tm_min then
          wattron iface.scr.timed_win A.underline
       else
          wattroff iface.scr.timed_win A.underline;
-      assert (mvwaddstr iface.scr.timed_win i 2 s)
+      assert (mvwaddstr iface.scr.timed_win i 2 ts_str);
+      Rcfile.color_off iface.scr.timed_win Rcfile.Timed_current;
+      wattroff iface.scr.timed_win A.bold;
+      let s = Str.string_before blank (iface.scr.tw_cols - 7) in
+      assert (mvwaddstr iface.scr.timed_win i 7 s)
    done;
    Rcfile.color_off iface.scr.timed_win Rcfile.Timed_default;
    wattroff iface.scr.timed_win (A.reverse lor A.underline);
+   (* draw in the timed reminders *)
    let rec process_reminders rem_list indent =
       Rcfile.color_on iface.scr.timed_win indent_colors.(indent);
       wattron iface.scr.timed_win A.bold;
       begin match rem_list with
       |[] ->
          ()
-      |(start, finish, msg, filename, line_num) :: tail ->
+      |(start, finish, msg, filename, line_num, has_weight) :: tail ->
          let rem_top_line =
             round_down ((start -. iface.top_timestamp) /. (time_inc iface))
          in
          let get_time_str () =
             if finish > start then
-               (desc_string_of_tm (Unix.localtime start)) ^ "-" ^
-               (desc_string_of_tm (Unix.localtime finish)) ^ " "
+               (desc_string_of_tm1 (Unix.localtime start)) ^ "-" ^
+               (desc_string_of_tm2 (Unix.localtime finish)) ^ " "
             else
-               (desc_string_of_tm (Unix.localtime start)) ^ " "
+               (desc_string_of_tm1 (Unix.localtime start)) ^ " "
          in
          (* draw the top line of a reminder *)
          let clock_pad = if !Rcfile.schedule_12_hour then 10 else 8 in
@@ -346,7 +366,7 @@ let draw_timed_window iface reminders top lines =
             else
                wattroff iface.scr.timed_win A.underline;
             iface.timed_lineinfo.(rem_top_line) <- 
-               (filename, line_num, time_str, msg) :: iface.timed_lineinfo.(rem_top_line);
+               (filename, line_num, time_str, msg, start) :: iface.timed_lineinfo.(rem_top_line);
             trunc_mvwaddstr iface.scr.timed_win rem_top_line (clock_pad + (9 * indent)) 
                (iface.scr.tw_cols - clock_pad - (9 * indent)) ("  " ^ msg);
             assert (mvwaddch iface.scr.timed_win rem_top_line 
@@ -375,7 +395,7 @@ let draw_timed_window iface reminders top lines =
                else
                   wattroff iface.scr.timed_win A.underline;
                iface.timed_lineinfo.(rem_top_line + !count) <- 
-                  (filename, line_num, time_str, msg) :: 
+                  (filename, line_num, time_str, msg, start) :: 
                   iface.timed_lineinfo.(rem_top_line + !count);
                trunc_mvwaddstr iface.scr.timed_win (rem_top_line + !count) 
                   (clock_pad + (9 * indent)) (iface.scr.tw_cols - clock_pad - (9 * indent)) " ";
@@ -405,13 +425,32 @@ let draw_timed_window iface reminders top lines =
 
 (* Draw the entire timed reminders window *)
 let draw_timed iface reminders = 
-   draw_timed_window iface reminders 0 iface.scr.tw_lines
+   draw_timed_window iface reminders 0 iface.scr.tw_lines;
+   {iface with last_timed_refresh = Unix.time ()}
+
+
+(* Draw just a portion of the timed window when possible.  If
+ * iface.last_timed_refresh indicates that the display is stale,
+ * then do a whole-window update.  This is to make sure that the
+ * current timeslot gets highlighted properly, and old highlights
+ * get erased. *)
+let draw_timed_try_window iface reminders top lines =
+   let curr_tm  = Unix.localtime (Unix.time ()) in
+   let timeslot = round_time iface.zoom_level curr_tm in
+   let (ts, _)  = Unix.mktime timeslot in
+   if iface.last_timed_refresh < ts then
+      draw_timed iface reminders
+   else begin
+      draw_timed_window iface reminders top lines;
+      iface
+   end
 
 
 (* render a calendar for the given reminders record *)
 let draw_calendar (iface : interface_state_t) 
        (reminders : three_month_rem_t) : unit =
-   let curr_tm = Unix.localtime (timestamp_of_line iface iface.left_selection) in
+   let sel_tm   = Unix.localtime (timestamp_of_line iface iface.left_selection)
+   and today_tm = Unix.localtime (Unix.time()) in
    let cal = reminders.curr_cal in
    let acs = get_acs_codes () in
    Rcfile.color_on iface.scr.calendar_win Rcfile.Right_divider;
@@ -424,11 +463,11 @@ let draw_calendar (iface : interface_state_t)
    wclrtoeol iface.scr.calendar_win;
    Rcfile.color_on iface.scr.calendar_win Rcfile.Calendar_labels;
    wattron iface.scr.calendar_win A.bold;
-   assert (waddstr iface.scr.calendar_win cal.title);
+   assert (waddstr iface.scr.calendar_win cal.Cal.title);
    wattroff iface.scr.calendar_win A.bold;
    assert (wmove iface.scr.calendar_win (vspacer + 1) hspacer);
    wclrtoeol iface.scr.calendar_win;
-   assert (waddstr iface.scr.calendar_win cal.weekdays);
+   assert (waddstr iface.scr.calendar_win cal.Cal.weekdays);
    Rcfile.color_off iface.scr.calendar_win Rcfile.Calendar_labels;
    (* draw the day numbers *)
    let ws = Str.regexp " " in
@@ -451,11 +490,28 @@ let draw_calendar (iface : interface_state_t)
                   assert (waddstr iface.scr.calendar_win s)
                |Str.Text d ->
                   let day = pred (int_of_string d) in
-                  if succ day = curr_tm.Unix.tm_mday then begin
+                  if succ day = sel_tm.Unix.tm_mday then begin
                      (* highlight selected day *)
-                     Rcfile.color_on iface.scr.calendar_win Rcfile.Calendar_today;
+                     if today_tm.Unix.tm_year = sel_tm.Unix.tm_year &&
+                        today_tm.Unix.tm_mon = sel_tm.Unix.tm_mon &&
+                        succ day = today_tm.Unix.tm_mday then
+                        (* if it's also today's date, use boldface *)
+                           wattron iface.scr.calendar_win A.bold
+                     else
+                        ();
+                     Rcfile.color_on iface.scr.calendar_win Rcfile.Calendar_selection;
                      assert (waddstr iface.scr.calendar_win d);
-                     Rcfile.color_off iface.scr.calendar_win Rcfile.Calendar_today
+                     Rcfile.color_off iface.scr.calendar_win Rcfile.Calendar_selection;
+                     wattroff iface.scr.calendar_win A.bold
+                  end else if today_tm.Unix.tm_year = sel_tm.Unix.tm_year &&
+                              today_tm.Unix.tm_mon = sel_tm.Unix.tm_mon &&
+                              succ day = today_tm.Unix.tm_mday then begin
+                     (* highlight today's date *)
+                     Rcfile.color_on iface.scr.calendar_win Rcfile.Calendar_today;
+                     wattron iface.scr.calendar_win A.bold;
+                     assert (waddstr iface.scr.calendar_win d);
+                     Rcfile.color_off iface.scr.calendar_win Rcfile.Calendar_today;
+                     wattroff iface.scr.calendar_win A.bold
                   end else if reminders.curr_counts.(day) <= !Rcfile.busy_level1 then begin
                      Rcfile.color_on iface.scr.calendar_win Rcfile.Calendar_level1;
                      assert (waddstr iface.scr.calendar_win d);
@@ -480,14 +536,15 @@ let draw_calendar (iface : interface_state_t)
                      assert (waddstr iface.scr.calendar_win d);
                      Rcfile.color_off iface.scr.calendar_win Rcfile.Calendar_level3;
                      wattroff iface.scr.calendar_win A.bold
-                  end
+                  end;
+                  wattroff iface.scr.calendar_win A.reverse
                end;
                draw_el days
          in
          draw_el split_week;
          draw_week tail (succ line)
    in
-   draw_week cal.days (vspacer + 2);
+   draw_week cal.Cal.days (vspacer + 2);
    assert (wnoutrefresh iface.scr.calendar_win)
 
 
@@ -521,7 +578,7 @@ let draw_untimed (iface : interface_state_t) reminders =
    Rcfile.color_on iface.scr.untimed_win Rcfile.Untimed_reminder;
    (* Filter the reminders list to find only those corresponding to the
     * selected day *)
-   let is_current (rem_ts, msg, filename, line_num) =
+   let is_current (rem_ts, msg, filename, line_num, has_weight) =
       rem_ts >= day_start_ts && rem_ts < day_end_ts
    in
    let today_reminders = List.filter is_current reminders in
@@ -531,7 +588,6 @@ let draw_untimed (iface : interface_state_t) reminders =
       iface.right_selection > List.length today_reminders then
          if List.length today_reminders = 0 then {
             iface with right_selection = 1;
-                       selected_side   = Left
          } else {
             iface with right_selection = List.length today_reminders
          }
@@ -541,8 +597,13 @@ let draw_untimed (iface : interface_state_t) reminders =
    let rec render_lines rem_list n line =
       match rem_list with
       |[] ->
-         ()
-      |(rem_ts, msg, filename, line_num) :: tail ->
+         if n = 0 && iface.selected_side = Right then begin
+            wattron iface.scr.untimed_win A.reverse;
+            trunc_mvwaddstr iface.scr.untimed_win line 2 (iface.scr.uw_cols - 3)  blank;
+            wattroff iface.scr.untimed_win A.reverse
+         end else
+            ()
+      |(rem_ts, msg, filename, line_num, has_weight) :: tail ->
          if line < iface.scr.uw_lines then
             if n >= iface.top_untimed then begin
                if line = iface.right_selection && iface.selected_side = Right then
@@ -657,11 +718,12 @@ let draw_msg iface =
          |[] -> 
             ([""], [["(no reminder selected)"]])
          |rem_list -> 
-              let get_times (_, _, time_str, msg) = time_str in
-              let get_lines (_, _, time_str, msg) = 
+              let sorted_rem_list = List.fast_sort sort_lineinfo rem_list in
+              let get_times (_, _, time_str, msg, start) = time_str in
+              let get_lines (_, _, time_str, msg, start) = 
                  word_wrap msg (iface.scr.mw_cols - 24)
               in
-              (List.rev_map get_times rem_list, List.rev_map get_lines rem_list)
+              (List.rev_map get_times sorted_rem_list, List.rev_map get_lines sorted_rem_list)
          end
       |Right ->
          begin match iface.untimed_lineinfo.(iface.right_selection) with
