@@ -92,7 +92,7 @@ let resize_subwins iface =
    let height, width  = get_size () in
    let cal_height     = 10
    and cal_width      = 40 in
-   let msg_height     = 5 in
+   let msg_height     = 6 in
    let err_height     = 1 in
    let timed_height   = height - 1 - msg_height - err_height
    and timed_width    = width - cal_width
@@ -100,25 +100,18 @@ let resize_subwins iface =
    and untimed_width  = cal_width in
    let new_scr = 
       if height >= 24 then 
-         if width >= 80 then {
-            stdscr       = iface.scr.stdscr;
+         if width >= 80 then {iface.scr with
             lines        = height;
             cols         = width;
-            help_win     = iface.scr.help_win;  (* subwindow pointers are unchanged *)
             hw_cols      = width;
-            timed_win    = iface.scr.timed_win;
             tw_lines     = timed_height;
             tw_cols      = timed_width;
-            calendar_win = iface.scr.calendar_win;
             cw_lines     = cal_height;
             cw_cols      = cal_width;
-            untimed_win  = iface.scr.untimed_win;
             uw_lines     = untimed_height;
             uw_cols      = untimed_width;
-            msg_win      = iface.scr.msg_win;
             mw_lines     = msg_height;
             mw_cols      = width;
-            err_win      = iface.scr.err_win;
             ew_lines     = err_height;
             ew_cols      = pred width
          }
@@ -150,10 +143,10 @@ let handle_refresh (iface : interface_state_t) reminders =
    let _ = touchwin iface.scr.untimed_win in
    let _ = touchwin iface.scr.msg_win in 
    draw_help iface;
-   draw_timed iface reminders.Remind.all_timed;
-   draw_date_strip iface;
-   draw_calendar iface reminders;
-   let new_iface = draw_untimed iface reminders.Remind.curr_untimed in
+   let new_iface = draw_timed iface reminders.Remind.all_timed in
+   draw_date_strip new_iface;
+   draw_calendar new_iface reminders;
+   let new_iface = draw_untimed new_iface reminders.Remind.curr_untimed in
    draw_error new_iface "" false;
    (new_iface, reminders)
    
@@ -165,7 +158,7 @@ let handle_resize (iface : interface_state_t) reminders =
    let resized_iface = {
       iface with scr = new_scr;
                  top_untimed = 0;
-                 left_selection = 1;
+                 left_selection = if !Rcfile.center_cursor then (iface.scr.tw_lines / 2) - 1 else 1;
                  right_selection = 1;
                  timed_lineinfo = Array.make new_scr.tw_lines []
    } in
@@ -183,7 +176,7 @@ let handle_selection_change iface reminders =
    let new_reminders = Remind.update_reminders reminders 
    (timestamp_of_line iface iface.left_selection) in
    let new_iface = draw_untimed iface new_reminders.Remind.curr_untimed in
-   draw_timed new_iface new_reminders.Remind.all_timed;
+   let new_iface = draw_timed new_iface new_reminders.Remind.all_timed in
    draw_date_strip new_iface;
    draw_calendar new_iface new_reminders;
    draw_error new_iface "" false;
@@ -201,13 +194,35 @@ let handle_selection_change_scroll iface reminders =
    (new_iface, new_reminders)
 
 
-(* handle a "scroll down" event when the timed window is focused *)
-let handle_scrolldown_timed (iface : interface_state_t) reminders =
-   let iface = {
-      iface with top_untimed = 0;
-                 top_desc = 0;
-                 right_selection = 1
+(* handle a "scroll down" event when the timed window is focused
+ * and the center_cursor option is turned on *)
+let handle_scrolldown_timed_center (iface : interface_state_t) reminders =
+   let second_timestamp = timestamp_of_line iface 1 in
+   let iface2 = {
+      iface with top_timestamp = second_timestamp
    } in
+   let (new_iface, new_reminders) =
+      handle_selection_change_scroll iface2 reminders
+   in
+   (* use a curses scroll operation to shift up the timed window *)
+   assert (wscrl new_iface.scr.timed_win 1);
+   (* adjust lineinfo array to compensate for scrolling *)
+   Array.blit iface.timed_lineinfo 1 iface.timed_lineinfo 0
+      (pred (Array.length iface.timed_lineinfo));
+   (* do a two-line update to recenter the cursor *)
+   let new_iface = draw_timed_try_window new_iface new_reminders.Remind.all_timed
+      (iface.left_selection - 1) 2 in
+   (* draw in the new line at the bottom of the screen *)
+   let new_iface = draw_timed_try_window new_iface new_reminders.Remind.all_timed
+      (iface.scr.tw_lines - 1) 1 in
+   draw_date_strip new_iface;
+   (new_iface, new_reminders)
+
+
+
+(* handle a "scroll down" event when the timed window is focused
+ * and the center_cursor option is turned off *)
+let handle_scrolldown_timed_nocenter (iface : interface_state_t) reminders =
    if iface.left_selection < pred iface.scr.tw_lines then begin
       (* case 1: only the cursor moves *)
       let iface2 = {
@@ -217,8 +232,8 @@ let handle_scrolldown_timed (iface : interface_state_t) reminders =
          handle_selection_change_scroll iface2 reminders
       in
       (* make a two-line update to erase and redraw the cursor *)
-      draw_timed_window new_iface new_reminders.Remind.all_timed 
-         iface.left_selection 2;
+      let new_iface = draw_timed_try_window new_iface new_reminders.Remind.all_timed 
+         iface.left_selection 2 in
       (new_iface, new_reminders)
    end else begin
       (* case 2: the entire timed window scrolls *)
@@ -236,13 +251,28 @@ let handle_scrolldown_timed (iface : interface_state_t) reminders =
          (pred (Array.length iface.timed_lineinfo));
       (* do a two-line update to erase the cursor and draw in the
        * new line at the bottom of the screen *)
-      draw_timed_window new_iface new_reminders.Remind.all_timed
-         (iface.scr.tw_lines - 2) 2;
+      let new_iface = draw_timed_try_window new_iface new_reminders.Remind.all_timed
+         (iface.scr.tw_lines - 2) 2 in
       draw_date_strip new_iface;
       (new_iface, new_reminders)
    end
 
 
+
+(* handle a "scroll down" event when the timed window is focused *)
+let handle_scrolldown_timed (iface : interface_state_t) reminders =
+   let iface = {
+      iface with top_untimed = 0;
+                 top_desc = 0;
+                 right_selection = 1
+   } in
+   if !Rcfile.center_cursor then
+      handle_scrolldown_timed_center iface reminders
+   else
+      handle_scrolldown_timed_nocenter iface reminders
+
+
+
 (* handle a "scroll down" event when the untimed window is focused *)
 let handle_scrolldown_untimed (iface : interface_state_t) reminders =
    if iface.right_selection < pred iface.scr.uw_lines then begin
@@ -267,13 +297,34 @@ let handle_scrolldown_untimed (iface : interface_state_t) reminders =
 
 
 
-(* handle a "scroll up" event when the timed window is focused *)
-let handle_scrollup_timed (iface : interface_state_t) reminders =
-   let iface = {
-      iface with top_untimed = 0;
-                 top_desc = 0;
-                 right_selection = 1
+(* handle a "scroll up" event when the timed window is focused
+ * and the center_cursor option is turned on *)
+let handle_scrollup_timed_center (iface : interface_state_t) reminders =
+   let prev_timestamp = timestamp_of_line iface (-1) in
+   let iface2 = {
+      iface with top_timestamp = prev_timestamp
    } in
+   let (new_iface, new_reminders) = 
+      handle_selection_change_scroll iface2 reminders
+   in
+   (* use a curses scroll operation to shift up the timed window *)
+   assert (wscrl new_iface.scr.timed_win (-1));
+   (* adjust lineinfo array to compensate for scrolling *)
+   Array.blit iface.timed_lineinfo 0 iface.timed_lineinfo 1
+      (pred (Array.length iface.timed_lineinfo));
+   (* do a two-line update to recenter the cursor *)
+   let new_iface = draw_timed_try_window new_iface new_reminders.Remind.all_timed 
+      iface.left_selection 2 in
+   (* draw in the new top line of the schedule *)
+   let new_iface = draw_timed_try_window new_iface new_reminders.Remind.all_timed 0 1 in
+   draw_date_strip new_iface;
+   (new_iface, new_reminders)
+
+
+
+(* handle a "scroll up" event when the timed window is focused
+ * and the center_cursor option is turned off *)
+let handle_scrollup_timed_nocenter (iface : interface_state_t) reminders =
    if iface.left_selection > 0 then begin
       (* case 1: only the cursor moves *)
       let iface2 = {
@@ -283,8 +334,8 @@ let handle_scrollup_timed (iface : interface_state_t) reminders =
          handle_selection_change_scroll iface2 reminders
       in
       (* make a two-line update to erase and redraw the cursor *)
-      draw_timed_window new_iface new_reminders.Remind.all_timed 
-         (pred iface.left_selection) 2;
+      let new_iface = draw_timed_try_window new_iface new_reminders.Remind.all_timed 
+         (pred iface.left_selection) 2 in
       (new_iface, new_reminders)
    end else begin
       (* case 2: the entire timed window scrolls *)
@@ -302,12 +353,26 @@ let handle_scrollup_timed (iface : interface_state_t) reminders =
          (pred (Array.length iface.timed_lineinfo));
       (* do a two-line update to erase the cursor and draw in the
        * new line at the bottom of the screen *)
-      draw_timed_window new_iface new_reminders.Remind.all_timed 0 2;
+      let new_iface = draw_timed_try_window new_iface new_reminders.Remind.all_timed 0 2 in
       draw_date_strip new_iface;
       (new_iface, new_reminders)
    end
 
 
+
+(* handle a "scroll up" event when the timed window is focused *)
+let handle_scrollup_timed (iface : interface_state_t) reminders =
+   let iface = {
+      iface with top_untimed = 0;
+                 top_desc = 0;
+                 right_selection = 1
+   } in
+   if !Rcfile.center_cursor then
+      handle_scrollup_timed_center iface reminders
+   else
+      handle_scrollup_timed_nocenter iface reminders
+
+
 (* handle a "scroll up" event when the untimed window is focused *)
 let handle_scrollup_untimed (iface : interface_state_t) reminders =
    if iface.right_selection > 1 then
@@ -397,22 +462,19 @@ let handle_zoom (iface : interface_state_t) reminders =
                        zoom_level    = Hour
          }
    in
-   draw_timed new_iface reminders.Remind.all_timed;
+   let new_iface = draw_timed new_iface reminders.Remind.all_timed in
    draw_date_strip new_iface;
    (new_iface, reminders)
 
 
 (* handle switching window focus *)
 let handle_switch_focus (iface : interface_state_t) reminders =
-   if iface.len_untimed > 0 then
-      let new_iface = 
-         match iface.selected_side with
-         |Left  -> {iface with selected_side = Right}
-         |Right -> {iface with selected_side = Left}
-      in
-      handle_selection_change new_iface reminders
-   else
-      (iface, reminders)
+   let new_iface = 
+      match iface.selected_side with
+      |Left  -> {iface with selected_side = Right}
+      |Right -> {iface with selected_side = Left}
+   in
+   handle_selection_change new_iface reminders
 
 
 (* handle switching to the current timeslot *)
@@ -420,10 +482,14 @@ let handle_home (iface : interface_state_t) reminders =
    let curr_time = Unix.localtime ((Unix.time ()) -. (time_inc iface)) in
    let (rounded_time, _) = Unix.mktime (round_time iface.zoom_level curr_time) in
    let new_iface = {
-      iface with top_timestamp   = rounded_time;
+      iface with top_timestamp = 
+                    if !Rcfile.center_cursor then
+                       rounded_time -. (time_inc iface) *. (float_of_int ((iface.scr.tw_lines / 2) - 2))
+                    else
+                       rounded_time -. (time_inc iface) *. 1.;
                  top_desc        = 0;
                  selected_side   = Left;
-                 left_selection  = 1;
+                 left_selection  = if !Rcfile.center_cursor then (iface.scr.tw_lines / 2) - 1 else 2;
                  right_selection = 1
    } in
    handle_selection_change new_iface reminders
@@ -444,12 +510,12 @@ let handle_new_reminder (iface : interface_state_t) reminders rem_type
             substitute_aux tail new_s
       in
       substitute_aux [
-         (Str.regexp "%M", Remind.string_of_tm_mon tm.Unix.tm_mon);
+         (Str.regexp "%M", Utility.string_of_tm_mon tm.Unix.tm_mon);
          (Str.regexp "%d", string_of_int tm.Unix.tm_mday);
          (Str.regexp "%y", string_of_int (tm.Unix.tm_year + 1900));
          (Str.regexp "%h", Printf.sprintf "%.2d" tm.Unix.tm_hour);
          (Str.regexp "%m", Printf.sprintf "%.2d" tm.Unix.tm_min);
-         (Str.regexp "%w", Remind.string_of_tm_wday tm.Unix.tm_wday)
+         (Str.regexp "%w", Utility.string_of_tm_wday tm.Unix.tm_wday)
       ] template
    in
    try
@@ -499,12 +565,11 @@ let handle_new_reminder (iface : interface_state_t) reminders rem_type
       end;
       let r = Remind.create_three_month iface.top_timestamp in
       (* if the untimed list has been altered, change the focus to
-       * the timed window *)
+       * the first element of the list *)
       let new_iface =
          if List.length r.Remind.curr_untimed <> 
             List.length reminders.Remind.curr_untimed then {
-            iface with selected_side = Left;
-                       top_untimed = 0;
+            iface with top_untimed = 0;
                        top_desc = 0;
                        right_selection = 1
             }
@@ -523,16 +588,27 @@ let handle_new_reminder (iface : interface_state_t) reminders rem_type
  * First find the date of the occurrence, by matching on the output of 'remind
  * -n'.  For that date, recompute timed and untimed reminder lists.  Filter the
  * timed and untimed reminder lists to get only the entries falling on the
- * occurrence date.  Search through the filtered timed list first, then the
- * filtered untimed list, and locate the first entry that matches the regex.
- * Finally, reconfigure the iface record to highlight the matched entry. *)
-let handle_find_next (iface : interface_state_t) reminders =
+ * occurrence date.  Search through the filtered timed list first, and locate
+ * the first entry that matches the regex (if any).  Then search through the 
+ * filtered untimed list, and locate the first entry that matches the regex (if any).
+ * If only one of the lists has a match, return that one; if both lists have a match,
+ * return the one with the earlier timestamp.
+ * Finally, reconfigure the iface record to highlight the matched entry.
+ *
+ * The saved search regex can be ignored by providing Some regex as the
+ * override_regex parameter. *)
+let handle_find_next (iface : interface_state_t) reminders override_regex =
+   let search_regex =
+      match override_regex with
+      |None       -> iface.search_regex
+      |Some regex -> regex
+   in
    try
       (* Note: selected_ts is the timestamp of the next timeslot, minus one second.
        *       This ensures that searching begins immediately after the current
        *       selection. *)
       let selected_ts     = (timestamp_of_line iface (succ iface.left_selection)) -. 1.0 in
-      let occurrence_time = Remind.find_next iface.search_regex selected_ts in
+      let occurrence_time = Remind.find_next search_regex selected_ts in
       let new_reminders   = Remind.update_reminders reminders occurrence_time in
       let occurrence_tm   = Unix.localtime occurrence_time in
       let temp1 = {
@@ -549,49 +625,82 @@ let handle_find_next (iface : interface_state_t) reminders =
       let (day_start_ts, _) = Unix.mktime temp1 in
       let (day_end_ts, _)   = Unix.mktime temp2 in
       (* filter functions to determine reminders falling on the occurrence day *)
-      let is_current_untimed (rem_ts, msg, _, _) =
+      let is_current_untimed (rem_ts, msg, _, _, _) =
          rem_ts >= day_start_ts && rem_ts < day_end_ts
       in
-      let is_current_timed (rem_ts, msg, _, _, _) =
+      let is_current_timed (rem_ts, msg, _, _, _, _) =
          rem_ts >= day_start_ts && rem_ts < day_end_ts
       in
       (* test the untimed reminders list for entries that match the regex *)
-      let rec check_untimed untimed n =
+      let rec check_untimed untimed n timed_match =
          match untimed with
          |[] ->
-            let _ = beep () in
-            draw_error iface "search expression not found." false;
-            assert (doupdate ());
-            (iface, reminders)
-         |(ts, msg, _, _) :: tail ->
+            begin match timed_match with
+            |None ->
+               let _ = beep () in
+               draw_error iface "search expression not found." false;
+               assert (doupdate ());
+               (iface, reminders)
+            |Some (timed_match_ts, timed_match_iface) ->
+               handle_selection_change timed_match_iface new_reminders
+            end
+         |(ts, msg, _, _, _) :: tail ->
             begin try
                if ts > selected_ts then
-                  let _ = Str.search_forward iface.search_regex msg 0 in
+                  let _ = Str.search_forward search_regex msg 0 in
                   let tm = Unix.localtime ts in
                   let (rounded_time, _) = Unix.mktime (round_time iface.zoom_level tm) in
                   let new_iface =
                      (* take care of highlighting the correct untimed reminder *)
-                     if n >= iface.scr.uw_lines then {
-                        iface with top_timestamp   = rounded_time;
-                                   top_untimed     = n - iface.scr.uw_lines + 1;
-                                   top_desc        = 0;
-                                   left_selection  = 0;
-                                   right_selection = pred iface.scr.uw_lines;
-                                   selected_side   = Right
-                     } else {
-                        iface with top_timestamp   = rounded_time;
-                                   top_untimed     = 0;
-                                   top_desc        = 0;
-                                   left_selection  = 0;
-                                   right_selection = n;
-                                   selected_side   = Right
-                     }
+                     if n >= iface.scr.uw_lines then 
+                        if !Rcfile.center_cursor then {
+                           iface with top_timestamp   = rounded_time -. (time_inc iface) *. 
+                                                        (float_of_int (iface.scr.tw_lines / 2 - 1));
+                                      top_untimed     = n - iface.scr.uw_lines + 1;
+                                      top_desc        = 0;
+                                      left_selection  = (iface.scr.tw_lines / 2) - 1;
+                                      right_selection = pred iface.scr.uw_lines;
+                                      selected_side   = Right
+                        } else {
+                           iface with top_timestamp   = rounded_time;
+                                      top_untimed     = n - iface.scr.uw_lines + 1;
+                                      top_desc        = 0;
+                                      left_selection  = 0;
+                                      right_selection = pred iface.scr.uw_lines;
+                                      selected_side   = Right
+                        }
+                     else 
+                        if !Rcfile.center_cursor then {
+                           iface with top_timestamp   = rounded_time -. (time_inc iface) *. 
+                                                        (float_of_int (iface.scr.tw_lines / 2 - 1));
+                                      top_untimed     = 0;
+                                      top_desc        = 0;
+                                      left_selection  = (iface.scr.tw_lines / 2) - 1 ;
+                                      right_selection = n;
+                                      selected_side   = Right
+                        } else {
+                           iface with top_timestamp   = rounded_time;
+                                      top_untimed     = 0;
+                                      top_desc        = 0;
+                                      left_selection  = 0;
+                                      right_selection = n;
+                                      selected_side   = Right
+                        }
                   in
-                  handle_selection_change new_iface new_reminders
+                  (* choose between the timed reminder match and the untimed reminder match *)
+                  begin match timed_match with
+                  |None ->
+                     handle_selection_change new_iface new_reminders
+                  |Some (timed_match_ts, timed_match_iface) ->
+                     if timed_match_ts < ts then
+                        handle_selection_change timed_match_iface new_reminders
+                     else
+                        handle_selection_change new_iface new_reminders
+                  end
                else
                   raise Not_found
             with Not_found ->
-               check_untimed tail (succ n)
+               check_untimed tail (succ n) timed_match
             end
       in
       (* test the timed reminders list for entries that match the regex *)
@@ -601,21 +710,34 @@ let handle_find_next (iface : interface_state_t) reminders =
             let today_untimed = 
                List.filter is_current_untimed new_reminders.Remind.curr_untimed
             in
-            check_untimed today_untimed 1
-         |(ts, _, msg, _, _) :: tail ->
+            check_untimed today_untimed 1 None
+         |(ts, _, msg, _, _, _) :: tail ->
             begin try
                if ts > selected_ts then
-                  let _ = Str.search_forward iface.search_regex msg 0 in
+                  let _ = Str.search_forward search_regex msg 0 in
                   let tm = Unix.localtime ts in
                   let (rounded_time, _) = Unix.mktime (round_time iface.zoom_level tm) in
-                  let new_iface = {
-                     iface with top_timestamp   = rounded_time;
-                                top_desc        = 0;
-                                left_selection  = 0;
-                                right_selection = 1;
-                                selected_side   = Left
-                  } in
-                  handle_selection_change new_iface new_reminders
+                  let new_iface = 
+                     if !Rcfile.center_cursor then {
+                        iface with top_timestamp   = rounded_time -. (time_inc iface) *. 
+                                                     (float_of_int (iface.scr.tw_lines / 2 - 1));
+                                   top_desc        = 0;
+                                   left_selection  = (iface.scr.tw_lines / 2) - 1;
+                                   right_selection = 1;
+                                   selected_side   = Left
+                        } 
+                     else {
+                        iface with top_timestamp   = rounded_time -. (time_inc iface) *. 2.;
+                                   top_desc        = 0;
+                                   left_selection  = 2;
+                                   right_selection = 1;
+                                   selected_side   = Left
+                     } 
+                  in
+                  let today_untimed = 
+                     List.filter is_current_untimed new_reminders.Remind.curr_untimed
+                  in
+                  check_untimed today_untimed 1 (Some (ts, new_iface))
                else
                   raise Not_found
             with Not_found ->
@@ -624,7 +746,7 @@ let handle_find_next (iface : interface_state_t) reminders =
       in
       let merged_rem = Remind.merge_timed new_reminders.Remind.curr_timed in
       check_timed (List.filter is_current_timed merged_rem)
-   with Not_found ->
+   with Remind.Occurrence_not_found ->
       let _ = beep () in
       draw_error iface "search expression not found." false;
       (iface, reminders)
@@ -639,14 +761,35 @@ let handle_begin_search (iface : interface_state_t) reminders =
    (new_iface, reminders)
 
 
-(* View the reminders for the selected date using 'less' *)
-let handle_view_reminders (iface : interface_state_t) reminders =
+(* Find the next reminder after the current selection.
+ * This algorithm is a dirty hack, but actually seems to work:
+ * 1) If an untimed reminder is selected, and if it is not the last
+ *    untimed reminder in the list, then highlight the next untimed reminder
+ * 2) Otherwise, run find_next with a regexp that matches anything. *)
+let handle_next_reminder (iface : interface_state_t) reminders =
+   if iface.selected_side = Right && 
+   iface.right_selection < iface.len_untimed - iface.top_untimed then 
+      handle_scrolldown_untimed iface reminders
+   else
+      handle_find_next iface reminders (Some (Str.regexp ""))
+
+
+(* View the reminders for the selected date using 'less'.
+ * If 'trigger_all' is true, then all nonexpired reminders are
+ * triggered. *)
+let handle_view_reminders (iface : interface_state_t) reminders trigger_all =
    let ts = timestamp_of_line iface iface.left_selection in
    let tm = Unix.localtime ts in
-   let rem_date_str = (Remind.string_of_tm_mon tm.Unix.tm_mon) ^ " " ^ 
+   let rem_date_str = (Utility.string_of_tm_mon tm.Unix.tm_mon) ^ " " ^ 
                       (string_of_int tm.Unix.tm_mday) ^ " " ^
                       (string_of_int (tm.Unix.tm_year + 1900)) in
-   let command = "remind -q -g " ^ !Rcfile.reminders_file ^ " " ^
+   let partial_command =
+      if trigger_all then
+         !Rcfile.remind_command ^ " -q -g -t "
+      else
+         !Rcfile.remind_command ^ " -q -g "
+   in
+   let command = partial_command ^ !Rcfile.reminders_file ^ " " ^
    rem_date_str ^ " | less -c" in
    def_prog_mode ();
    endwin ();
@@ -660,15 +803,28 @@ let handle_view_reminders (iface : interface_state_t) reminders =
    handle_refresh iface reminders
 
 
-(* View all non-expired reminders for the selected date using 'less' *)
-let handle_view_all_reminders (iface : interface_state_t) reminders =
+
+(* View Remind's formatted calendar output for the selected date.  If
+ * 'week_only' is true, then create a week calendar, otherwise a month
+ * calendar. *)
+let handle_view_calendar (iface : interface_state_t) reminders week_only =
    let ts = timestamp_of_line iface iface.left_selection in
    let tm = Unix.localtime ts in
-   let rem_date_str = (Remind.string_of_tm_mon tm.Unix.tm_mon) ^ " " ^ 
+   let rem_date_str = (Utility.string_of_tm_mon tm.Unix.tm_mon) ^ " " ^ 
                       (string_of_int tm.Unix.tm_mday) ^ " " ^
                       (string_of_int (tm.Unix.tm_year + 1900)) in
-   let command = "remind -q -g -t " ^ !Rcfile.reminders_file ^ " " ^
-   rem_date_str ^ " | less -c" in
+   let partial_command = 
+      if week_only then
+         Printf.sprintf "%s -c+1 -w%d " !Rcfile.remind_command iface.scr.cols
+      else
+         Printf.sprintf "%s -c -w%d " !Rcfile.remind_command iface.scr.cols
+   in
+   let time_option = if !Rcfile.description_12_hour then "-b0 " else "-b1 " in
+   let weekday_option = if !Rcfile.week_starts_monday then "-m " else "" in
+   let command = 
+      partial_command ^ time_option ^ weekday_option ^ 
+      !Rcfile.reminders_file ^ " " ^ rem_date_str ^ " | less -c" 
+   in
    def_prog_mode ();
    endwin ();
    let _ = Unix.system command in 
@@ -681,6 +837,42 @@ let handle_view_all_reminders (iface : interface_state_t) reminders =
    handle_refresh iface reminders
 
 
+let handle_view_keybindings (iface : interface_state_t) reminders =
+   let bindings = ref [] in
+   let find_binding commandstr operation =
+      match operation with
+      |Rcfile.CommandOp command ->
+         begin try
+            let key_str = Hashtbl.find Rcfile.table_command_key command in
+            bindings := (Printf.sprintf "%-30s%-20s\n" commandstr key_str) :: !bindings
+         with Not_found ->
+            bindings := (Printf.sprintf "%-30s\n" commandstr) :: !bindings
+         end
+      |Rcfile.EntryOp entry ->
+         begin try
+            let key_str = Hashtbl.find Rcfile.table_entry_key entry in
+            bindings := (Printf.sprintf "%-30s%-20s\n" commandstr key_str) :: !bindings
+         with Not_found ->
+            bindings := (Printf.sprintf "%-30s\n" commandstr) :: !bindings
+         end
+   in
+   Hashtbl.iter find_binding Rcfile.table_commandstr_command;
+   let sorted_list = List.fast_sort Pervasives.compare !bindings in
+   let out_channel = open_out Rcfile.tmpfile in
+   List.iter (output_string out_channel) sorted_list;
+   close_out out_channel;
+   def_prog_mode ();
+   endwin ();
+   let _ = Unix.system ("less " ^ Rcfile.tmpfile) in 
+   reset_prog_mode ();
+   begin try
+      assert (curs_set 0)
+   with _ ->
+      ()
+   end;
+   handle_refresh iface reminders
+
+
 (* Handle scrolling down during selection dialog loop *)
 let handle_selection_dialog_scrolldown (elements : string list) 
        (selection : int) (top : int) =
@@ -756,24 +948,30 @@ let handle_edit (iface : interface_state_t) reminders =
          begin match iface.timed_lineinfo.(iface.left_selection) with
          | [] -> 
             None
-         | (f, l, t, m) :: [] -> 
+         | (f, l, t, m, s) :: [] -> 
             Some (f, l, m)
          | rem_list -> 
-              let get_msg (_, _, time, msg) = (adjust_s 16 time) ^ msg in
-              let msg_list = List.rev_map get_msg rem_list in
+              let sorted_rem_list = List.fast_sort sort_lineinfo rem_list in
+              let get_msg (_, _, time, msg, _) = (adjust_s 16 time) ^ msg in
+              let msg_list = List.rev_map get_msg sorted_rem_list in
               let selected_msg =
                  do_selection_dialog iface "Choose a reminder to edit" msg_list
               in
-              let test_msg_match (_, _, time, msg) = 
+              let test_msg_match (_, _, time, msg, _) = 
                  ((adjust_s 16 time) ^ msg) = selected_msg in
-              let (f, l, t, m) = (List.find test_msg_match rem_list) in
+              let (f, l, t, m, s) = (List.find test_msg_match sorted_rem_list) in
               Some (f, l, m)
          end
       |Right -> iface.untimed_lineinfo.(iface.right_selection)
    in
    begin match fl with
    |None ->
-      (iface, reminders)
+      if iface.selected_side = Left then
+         handle_new_reminder iface reminders Timed
+         (Utility.expand_file !Rcfile.reminders_file)
+      else
+         handle_new_reminder iface reminders Untimed
+         (Utility.expand_file !Rcfile.reminders_file)
    |Some (filename, line_num, msg) -> 
       let filename_sub = Str.regexp "%f" in
       let lineno_sub   = Str.regexp "%n" in
@@ -791,13 +989,13 @@ let handle_edit (iface : interface_state_t) reminders =
          ()
       end;
       let r = Remind.create_three_month iface.top_timestamp in
-      (* if the untimed list has been altered, change the focus to
-       * the timed window *)
+      (* if the untimed list has been altered, change the focus
+       * to the first element *)
       let new_iface =
          if List.length r.Remind.curr_untimed <> 
             List.length reminders.Remind.curr_untimed then {
-            iface with selected_side = Left;
-                       top_untimed = 0;
+            iface with top_untimed = 0;
+                       top_desc = 0;
                        right_selection = 1
             }
          else
@@ -807,6 +1005,39 @@ let handle_edit (iface : interface_state_t) reminders =
    end
 
 
+
+(* handle free editing of the reminders file *)
+let handle_edit_any (iface : interface_state_t) reminders remfile =
+   let filename_sub = Str.regexp "%f" in
+   let command = 
+      Str.global_replace filename_sub remfile !Rcfile.edit_any_command
+   in
+   def_prog_mode ();
+   endwin ();
+   let _ = Unix.system command in 
+   reset_prog_mode ();
+   begin try
+      assert (curs_set 0)
+   with _ ->
+      ()
+   end;
+   let r = Remind.create_three_month iface.top_timestamp in
+   (* if the untimed list has been altered, change the focus
+    * to the first element *)
+   let new_iface =
+      if List.length r.Remind.curr_untimed <> 
+         List.length reminders.Remind.curr_untimed then {
+         iface with top_untimed = 0;
+                    top_desc = 0;
+                    right_selection = 1
+         }
+      else
+         iface
+   in
+   handle_refresh new_iface r
+
+
+
 (* handle scrolling the description window up *)
 let handle_scroll_desc_up iface reminders =
    let new_iface = {iface with top_desc = succ iface.top_desc} in
@@ -857,6 +1088,12 @@ let handle_keypress key (iface : interface_state_t) reminders =
             handle_home iface reminders
          |Rcfile.Edit ->
             handle_edit iface reminders
+         |Rcfile.EditAny ->
+            let remfile = 
+               do_selection_dialog iface "Choose a reminders file"
+               (Remind.get_included_remfiles ())
+            in
+            handle_edit_any iface reminders remfile
          |Rcfile.ScrollDescUp ->
             handle_scroll_desc_up iface reminders
          |Rcfile.ScrollDescDown ->
@@ -889,13 +1126,21 @@ let handle_keypress key (iface : interface_state_t) reminders =
             in
             handle_new_reminder iface reminders (General x) remfile
          |Rcfile.SearchNext ->
-            handle_find_next iface reminders
+            handle_find_next iface reminders None
          |Rcfile.BeginSearch ->
             handle_begin_search iface reminders
+         |Rcfile.NextReminder ->
+            handle_next_reminder iface reminders
          |Rcfile.ViewReminders ->
-            handle_view_reminders iface reminders
+            handle_view_reminders iface reminders false
          |Rcfile.ViewAllReminders ->
-            handle_view_all_reminders iface reminders
+            handle_view_reminders iface reminders true
+         |Rcfile.ViewWeek ->
+            handle_view_calendar iface reminders true
+         |Rcfile.ViewMonth ->
+            handle_view_calendar iface reminders false
+         |Rcfile.ViewKeybindings ->
+            handle_view_keybindings iface reminders
          |Rcfile.Refresh ->
             (* NOTE: I'm not sure why the endwin call is necessary here,
              * but I'm having problems getting a true full-screen refresh
@@ -929,7 +1174,7 @@ let handle_keypress key (iface : interface_state_t) reminders =
                              search_input = "";
                              is_entering_search = false
                } in
-               handle_find_next new_iface reminders
+               handle_find_next new_iface reminders None
             with Failure err ->
                let new_iface = {
                   iface with search_input = "";
@@ -961,12 +1206,16 @@ let handle_keypress key (iface : interface_state_t) reminders =
          end
       with Not_found ->
          begin try
-            let c = char_of_int key in
-            let new_iface = {
-               iface with search_input = iface.search_input ^ (String.make 1 c)
-            } in
-            draw_error new_iface ("search expression: " ^ new_iface.search_input) true;
-            (new_iface, reminders)
+            (* only printable characters are accepted for search strings *)
+            if key >= 32 && key <= 126 then begin
+               let c = char_of_int key in
+               let new_iface = {
+                  iface with search_input = iface.search_input ^ (String.make 1 c)
+               } in
+               draw_error new_iface ("search expression: " ^ new_iface.search_input) true;
+               (new_iface, reminders)
+            end else
+               failwith "cannot search for unprintable characters"
          with Failure _ ->
             let _ = beep () in
             (iface, reminders)
@@ -1025,9 +1274,9 @@ let run (iface : interface_state_t) =
    assert (keypad iface.scr.help_win true);
    draw_help iface;
    draw_date_strip iface;
-   draw_timed iface reminders.Remind.all_timed;
-   draw_calendar iface reminders;
-   let new_iface = draw_untimed iface reminders.Remind.curr_untimed in
+   let new_iface = draw_timed iface reminders.Remind.all_timed in
+   draw_calendar new_iface reminders;
+   let new_iface = draw_untimed new_iface reminders.Remind.curr_untimed in
    let new_iface = draw_msg new_iface in
    draw_error new_iface "" false;
    assert (doupdate ());
