@@ -40,21 +40,23 @@ let config_failwith s = raise (Config_failure s)
 
 type command_t = | ScrollUp | ScrollDown | NextDay | PrevDay 
                  | NextWeek | PrevWeek | NextMonth | PrevMonth
-                 | Home | Zoom | Edit | NewTimed | NewUntimed 
+                 | Home | Zoom | Edit | EditAny | NewTimed | NewUntimed 
                  | NewTimedDialog | NewUntimedDialog | SwitchWindow 
                  | SearchNext | BeginSearch | Quit | ViewReminders
                  | ScrollDescUp | ScrollDescDown | Refresh
-                 | ViewAllReminders | NewGenReminder of int
-                 | NewGenReminderDialog of int
+                 | ViewAllReminders | ViewWeek | ViewMonth
+                 | NextReminder | ViewKeybindings
+                 | NewGenReminder of int | NewGenReminderDialog of int
 
 type entry_operation_t = | EntryComplete | EntryBackspace | EntryExit
 
 type operation_t = CommandOp of command_t | EntryOp of entry_operation_t
 
-type colorable_object_t = | Help | Timed_default | Untimed_reminder
+type colorable_object_t = | Help | Timed_default | Timed_current | Untimed_reminder
                           | Timed_date | Selection_info | Description | Status 
                           | Calendar_labels | Calendar_level1 | Calendar_level2
-                          | Calendar_level3 | Calendar_today | Left_divider | Right_divider 
+                          | Calendar_level3 | Calendar_selection | Calendar_today 
+                          | Left_divider | Right_divider 
                           | Timed_reminder1 | Timed_reminder2 | Timed_reminder3
                           | Timed_reminder4
 
@@ -67,11 +69,17 @@ let table_command_key = Hashtbl.create 20
 let table_key_entry = Hashtbl.create 20
 let table_entry_key = Hashtbl.create 20
 
+let table_commandstr_command = Hashtbl.create 30
+let table_command_commandstr = Hashtbl.create 30
+
+(* Default Remind command *)
+let remind_command = ref "remind"
 (* Default reminders file *)
 let reminders_file = ref "~/.reminders"
 (* Default editing command strings *)
-let edit_old_command = ref "vim +%n + %f"
+let edit_old_command = ref "vim +%n %f"
 let edit_new_command = ref "vim -c '$' %f"
+let edit_any_command = ref "vim %f"
 (* Default new reminder templates *)
 let timed_template   = ref "REM %M %d %y AT %h:%m DURATION 1:00 MSG "
 let untimed_template = ref "REM %M %d %y MSG "
@@ -85,6 +93,10 @@ let template6        = ref None
 let template7        = ref None
 let template8        = ref None
 let template9        = ref None
+(* algorithm to use for counting busy-ness *)
+let busy_algorithm = ref 1
+(* number of minutes to assume an untimed reminder requires *)
+let untimed_duration = ref 60.
 (* Default thresholds for calendar colorization *)
 let busy_level1 = ref 2
 let busy_level2 = ref 4
@@ -97,6 +109,8 @@ let schedule_12_hour    = ref false
 let selection_12_hour   = ref true
 let status_12_hour      = ref true
 let description_12_hour = ref true
+(* Center the schedule on the cursor? *)
+let center_cursor = ref false
 (* List of included rc files *)
 let included_rcfiles : (string list) ref = ref []
 
@@ -106,6 +120,9 @@ let inverse_color_table = Hashtbl.create 20
 (* Final hash table that maps from object to color_pair index *)
 let object_palette      = Hashtbl.create 20
 
+let tmpfile = "/tmp/wyrd-tmp." ^ (string_of_int (Unix.getuid ()))
+      
+
 (* Turn colors on and off *)
 let color_on win obj =
    try
@@ -268,59 +285,74 @@ let unregister_binding key_string =
    end
 
 
+let commands_list = [
+   ("scroll_up"               , CommandOp ScrollUp);
+   ("scroll_down"             , CommandOp ScrollDown);
+   ("next_day"                , CommandOp NextDay);
+   ("previous_day"            , CommandOp PrevDay);
+   ("next_week"               , CommandOp NextWeek);
+   ("previous_week"           , CommandOp PrevWeek);
+   ("next_month"              , CommandOp NextMonth);
+   ("previous_month"          , CommandOp PrevMonth);
+   ("home"                    , CommandOp Home);
+   ("zoom"                    , CommandOp Zoom);
+   ("edit"                    , CommandOp Edit);
+   ("edit_any"                , CommandOp EditAny);
+   ("scroll_description_up"   , CommandOp ScrollDescUp);
+   ("scroll_description_down" , CommandOp ScrollDescDown);
+   ("new_timed"               , CommandOp NewTimed);
+   ("new_timed_dialog"        , CommandOp NewTimedDialog);
+   ("new_untimed"             , CommandOp NewUntimed);
+   ("new_untimed_dialog"      , CommandOp NewUntimedDialog);
+   ("new_template0"           , CommandOp (NewGenReminder 0));
+   ("new_template0_dialog"    , CommandOp (NewGenReminderDialog 0));
+   ("new_template1"           , CommandOp (NewGenReminder 1));
+   ("new_template1_dialog"    , CommandOp (NewGenReminderDialog 1));
+   ("new_template2"           , CommandOp (NewGenReminder 2));
+   ("new_template2_dialog"    , CommandOp (NewGenReminderDialog 2));
+   ("new_template3"           , CommandOp (NewGenReminder 3));
+   ("new_template3_dialog"    , CommandOp (NewGenReminderDialog 3));
+   ("new_template4"           , CommandOp (NewGenReminder 4));
+   ("new_template4_dialog"    , CommandOp (NewGenReminderDialog 4));
+   ("new_template5"           , CommandOp (NewGenReminder 5));
+   ("new_template5_dialog"    , CommandOp (NewGenReminderDialog 5));
+   ("new_template6"           , CommandOp (NewGenReminder 6));
+   ("new_template6_dialog"    , CommandOp (NewGenReminderDialog 6));
+   ("new_template7"           , CommandOp (NewGenReminder 7));
+   ("new_template7_dialog"    , CommandOp (NewGenReminderDialog 7));
+   ("new_template8"           , CommandOp (NewGenReminder 8));
+   ("new_template8_dialog"    , CommandOp (NewGenReminderDialog 8));
+   ("new_template9"           , CommandOp (NewGenReminder 9));
+   ("new_template9_dialog"    , CommandOp (NewGenReminderDialog 9));
+   ("switch_window"           , CommandOp SwitchWindow);
+   ("search_next"             , CommandOp SearchNext);
+   ("begin_search"            , CommandOp BeginSearch);
+   ("next_reminder"           , CommandOp NextReminder);
+   ("view_remind"             , CommandOp ViewReminders);
+   ("view_remind_all"         , CommandOp ViewAllReminders);
+   ("view_week"               , CommandOp ViewWeek);
+   ("view_month"              , CommandOp ViewMonth);
+   ("refresh"                 , CommandOp Refresh );
+   ("help"                    , CommandOp ViewKeybindings);
+   ("entry_complete"          , EntryOp EntryComplete);
+   ("entry_backspace"         , EntryOp EntryBackspace);
+   ("entry_cancel"            , EntryOp EntryExit);
+   ("quit"                    , CommandOp Quit)
+] in
+let create_translation ((commandstr, operation) : string * operation_t) =
+   Hashtbl.add table_commandstr_command commandstr operation;
+   Hashtbl.add table_command_commandstr operation commandstr
+in
+List.iter create_translation commands_list
+
+
 (* translate a command string to the command type it represents *)
 let operation_of_string command_str =
-   begin match command_str with
-   |"scroll_up"               -> CommandOp ScrollUp
-   |"scroll_down"             -> CommandOp ScrollDown
-   |"next_day"                -> CommandOp NextDay
-   |"previous_day"            -> CommandOp PrevDay
-   |"next_week"               -> CommandOp NextWeek
-   |"previous_week"           -> CommandOp PrevWeek
-   |"next_month"              -> CommandOp NextMonth
-   |"previous_month"          -> CommandOp PrevMonth
-   |"home"                    -> CommandOp Home
-   |"zoom"                    -> CommandOp Zoom
-   |"edit"                    -> CommandOp Edit
-   |"scroll_description_up"   -> CommandOp ScrollDescUp
-   |"scroll_description_down" -> CommandOp ScrollDescDown
-   |"new_timed"               -> CommandOp NewTimed
-   |"new_timed_dialog"        -> CommandOp NewTimedDialog
-   |"new_untimed"             -> CommandOp NewUntimed
-   |"new_untimed_dialog"      -> CommandOp NewUntimedDialog
-   |"new_template0"           -> CommandOp (NewGenReminder 0)
-   |"new_template0_dialog"    -> CommandOp (NewGenReminderDialog 0)
-   |"new_template1"           -> CommandOp (NewGenReminder 1)
-   |"new_template1_dialog"    -> CommandOp (NewGenReminderDialog 1)
-   |"new_template2"           -> CommandOp (NewGenReminder 2)
-   |"new_template2_dialog"    -> CommandOp (NewGenReminderDialog 2)
-   |"new_template3"           -> CommandOp (NewGenReminder 3)
-   |"new_template3_dialog"    -> CommandOp (NewGenReminderDialog 3)
-   |"new_template4"           -> CommandOp (NewGenReminder 4)
-   |"new_template4_dialog"    -> CommandOp (NewGenReminderDialog 4)
-   |"new_template5"           -> CommandOp (NewGenReminder 5)
-   |"new_template5_dialog"    -> CommandOp (NewGenReminderDialog 5)
-   |"new_template6"           -> CommandOp (NewGenReminder 6)
-   |"new_template6_dialog"    -> CommandOp (NewGenReminderDialog 6)
-   |"new_template7"           -> CommandOp (NewGenReminder 7)
-   |"new_template7_dialog"    -> CommandOp (NewGenReminderDialog 7)
-   |"new_template8"           -> CommandOp (NewGenReminder 8)
-   |"new_template8_dialog"    -> CommandOp (NewGenReminderDialog 8)
-   |"new_template9"           -> CommandOp (NewGenReminder 9)
-   |"new_template9_dialog"    -> CommandOp (NewGenReminderDialog 9)
-   |"switch_window"           -> CommandOp SwitchWindow
-   |"search_next"             -> CommandOp SearchNext
-   |"begin_search"            -> CommandOp BeginSearch
-   |"view_remind"             -> CommandOp ViewReminders
-   |"view_remind_all"         -> CommandOp ViewAllReminders
-   |"refresh"                 -> CommandOp Refresh 
-   |"entry_complete"          -> EntryOp EntryComplete
-   |"entry_backspace"         -> EntryOp EntryBackspace
-   |"entry_cancel"            -> EntryOp EntryExit
-   |"quit"                    -> CommandOp Quit
-   |_                         -> config_failwith ("Unknown command name \"" ^ command_str ^ "\"")
-   end
+   Hashtbl.find table_commandstr_command command_str
 
+(* translate a command to a string representation *)
+let string_of_operation op =
+   Hashtbl.find table_command_commandstr op
 
 
 (* Parse a line from a configuration file.  This operates on a stream
@@ -433,12 +465,16 @@ let parse_line line_stream =
       end
    | [< 'Kwd "set" >] ->
       begin match line_stream with parser
+      | [< 'Ident "remind_command" >] ->
+         parse_set "remind_command" remind_command (fun x -> x) "Expected a command string after "
       | [< 'Ident "reminders_file" >] ->
          parse_set "reminders_file" reminders_file (fun x -> x) "Expected a filename string after "
       | [< 'Ident "edit_old_command" >] ->
          parse_set "edit_old_command" edit_old_command (fun x -> x) "Expected a command string after "
       | [< 'Ident "edit_new_command" >] ->
          parse_set "edit_new_command" edit_new_command (fun x -> x) "Expected a command string after "
+      | [< 'Ident "edit_any_command" >] ->
+         parse_set "edit_any_command" edit_any_command (fun x -> x) "Expected a command string after "
       | [< 'Ident "timed_template" >] ->
          parse_set "timed_template" timed_template (fun x -> x) "Expected a template string after "
       | [< 'Ident "untimed_template" >] ->
@@ -463,6 +499,10 @@ let parse_line line_stream =
          parse_set "template8" template8 (fun x -> Some x) "Expected a template string after "
       | [< 'Ident "template9" >] ->
          parse_set "template9" template9 (fun x -> Some x) "Expected a template string after "
+      | [< 'Ident "busy_algorithm" >] ->
+         parse_set "busy_algorithm" busy_algorithm int_of_string "Expected an integral string after "
+      | [< 'Ident "untimed_duration" >] ->
+         parse_set "untimed_duration" untimed_duration float_of_string "Expected a float string after "
       | [< 'Ident "busy_level1" >] ->
          parse_set "busy_level1" busy_level1 int_of_string "Expected an integral string after "
       | [< 'Ident "busy_level2" >] ->
@@ -481,6 +521,8 @@ let parse_line line_stream =
          parse_set "status_12_hour" status_12_hour bool_of_string "Expected a boolean string after "
       | [< 'Ident "description_12_hour" >] ->
          parse_set "description_12_hour" description_12_hour bool_of_string "Expected a boolean string after "
+      | [< 'Ident "center_cursor" >] ->
+         parse_set "center_cursor" center_cursor bool_of_string "Expected a boolean string after "
       | [< >] ->
          config_failwith ("Unmatched variable name after \"set\"")
       end
@@ -488,6 +530,7 @@ let parse_line line_stream =
       begin match line_stream with parser
       | [< 'Ident "help" >]             -> parse_color "help" Help
       | [< 'Ident "timed_default" >]    -> parse_color "timed_default" Timed_default
+      | [< 'Ident "timed_current" >]    -> parse_color "timed_current" Timed_current
       | [< 'Ident "timed_reminder" >]   -> config_failwith 
              ("\"timed_reminder\" has been deprecated.  Please use \"timed_reminder1\".")
       | [< 'Ident "untimed_reminder" >] -> parse_color "untimed_reminder" Untimed_reminder
@@ -499,6 +542,7 @@ let parse_line line_stream =
       | [< 'Ident "calendar_level1" >]  -> parse_color "calendar_level1" Calendar_level1
       | [< 'Ident "calendar_level2" >]  -> parse_color "calendar_level2" Calendar_level2
       | [< 'Ident "calendar_level3" >]  -> parse_color "calendar_level3" Calendar_level3
+      | [< 'Ident "calendar_selection" >] -> parse_color "calendar_selection" Calendar_selection
       | [< 'Ident "calendar_today" >]   -> parse_color "calendar_today" Calendar_today
       | [< 'Ident "left_divider" >]     -> parse_color "left_divider" Left_divider
       | [< 'Ident "right_divider" >]    -> parse_color "right_divider" Right_divider
