@@ -22,92 +22,31 @@
  * functions for interfacing with 'remind(1)' *)
 
 
-exception String_of_tm_mon_failure of string
-exception String_of_tm_wday_failure of string
 exception Occurrence_not_found
 
-
-type cal_t = {
-   title    : string;
-   weekdays : string;
-   days     : string list
-}
+open Utility
 
 
 (* Storage for a three-month window of reminders and
  * the calendar for the current month.
  * Makes it possible to handle edge effects of moving
  * from month to month without constantly calling 
- * rem(1) and cal(1). *)
+ * rem(1) and regenerating calendar layouts. *)
 type three_month_rem_t = {
    curr_timestamp : float;
-   prev_timed     : (float * float * string * string * string) list array;
-   curr_timed     : (float * float * string * string * string) list array;
-   next_timed     : (float * float * string * string * string) list array;
-   all_timed      : (float * float * string * string * string) list array;
-   prev_untimed   : (float * string * string * string) list;
-   curr_untimed   : (float * string * string * string) list;
-   next_untimed   : (float * string * string * string) list;
-   all_untimed    : (float * string * string * string) list;
+   prev_timed     : (float * float * string * string * string * bool) list array;
+   curr_timed     : (float * float * string * string * string * bool) list array;
+   next_timed     : (float * float * string * string * string * bool) list array;
+   all_timed      : (float * float * string * string * string * bool) list array;
+   prev_untimed   : (float * string * string * string * bool) list;
+   curr_untimed   : (float * string * string * string * bool) list;
+   next_untimed   : (float * string * string * string * bool) list;
+   all_untimed    : (float * string * string * string * bool) list;
    curr_counts    : int array;
-   curr_cal       : cal_t
+   curr_cal       : Cal.t
 }
 
 
-let string_of_tm_mon i =
-   match i with
-   | 0 -> "Jan"
-   | 1 -> "Feb"
-   | 2 -> "Mar"
-   | 3 -> "Apr"
-   | 4 -> "May"
-   | 5 -> "Jun"
-   | 6 -> "Jul"
-   | 7 -> "Aug"
-   | 8 -> "Sep"
-   | 9 -> "Oct"
-   |10 -> "Nov"
-   |11 -> "Dec"
-   | x -> raise (String_of_tm_mon_failure ("unknown month " ^ (string_of_int x)))
-
-let full_string_of_tm_mon i =
-   match i with
-   | 0 -> "January"
-   | 1 -> "February"
-   | 2 -> "March"
-   | 3 -> "April"
-   | 4 -> "May"
-   | 5 -> "June"
-   | 6 -> "Jul"
-   | 7 -> "August"
-   | 8 -> "September"
-   | 9 -> "October"
-   |10 -> "November"
-   |11 -> "December"
-   | x -> raise (String_of_tm_mon_failure ("unknown month " ^ (string_of_int x)))
-
-let string_of_tm_wday i =
-   match i with
-   | 0 -> "Sun"
-   | 1 -> "Mon"
-   | 2 -> "Tuey"
-   | 3 -> "Wed"
-   | 4 -> "Thu"
-   | 5 -> "Fri"
-   | 6 -> "Sat"
-   | x -> raise (String_of_tm_wday_failure ("unknown weekday " ^ (string_of_int x)))
-
-let full_string_of_tm_wday i =
-   match i with
-   | 0 -> "Sunday"
-   | 1 -> "Monday"
-   | 2 -> "Tuesday"
-   | 3 -> "Wednesday"
-   | 4 -> "Thursday"
-   | 5 -> "Friday"
-   | 6 -> "Saturday"
-   | x -> raise (String_of_tm_wday_failure ("unknown weekday " ^ (string_of_int x)))
-
 
 (* Obtain two lists of reminders for the month of the timestamp argument.
  * The first list is for timed reminders, the second is for untimed. 
@@ -123,12 +62,14 @@ let full_string_of_tm_wday i =
  * month borders. *)
 let month_reminders timestamp =
    let comment_regex = Str.regexp "^#.*fileinfo \\([^ ]+\\) \\(.*\\)$" in
-   let rem_regex = Str.regexp "\\([^ ]+\\) [^ ]+ [^ ]+ \\([^ ]+\\) \\([^ ]+\\) \\(.*\\)$" in
+   let rem_regex = Str.regexp "\\([^ ]+\\) [^ ]+ \\([^ ]+\\) \\([^ ]+\\) \\([^ ]+\\) \\(.*\\)$" in
+   let nodisplay_regex = Str.regexp_case_fold ".*nodisplay" in
+   let noweight_regex = Str.regexp_case_fold ".*noweight" in
    let tm = Unix.localtime timestamp in
    let rem_date_str = (string_of_tm_mon tm.Unix.tm_mon) ^ " " ^ 
                       (string_of_int tm.Unix.tm_mday) ^ " " ^
                       (string_of_int (tm.Unix.tm_year + 1900)) in
-   let remind_channel = Unix.open_process_in ("remind -s -l -g -b2 " ^ 
+   let remind_channel = Unix.open_process_in (!Rcfile.remind_command ^ " -s -l -g -b2 " ^ 
    !Rcfile.reminders_file ^ " " ^ rem_date_str) in
    let num_indentations = 4 in
    let indentations = Array.make_matrix (24 * 31) num_indentations false in
@@ -156,9 +97,10 @@ let month_reminders timestamp =
             let rem_line   = input_line remind_channel in
             if Str.string_match rem_regex rem_line 0 then begin
                let date_s     = Str.matched_group 1 rem_line
-               and duration_s = Str.matched_group 2 rem_line
-               and min_s      = Str.matched_group 3 rem_line
-               and msg        = Str.matched_group 4 rem_line in
+               and tag        = Str.matched_group 2 rem_line
+               and duration_s = Str.matched_group 3 rem_line
+               and min_s      = Str.matched_group 4 rem_line
+               and msg        = Str.matched_group 5 rem_line in
                (* further subdivide the date string *)
                let date_arr = Array.of_list (Str.split (Str.regexp "/") date_s) in
                let year     = int_of_string date_arr.(0)
@@ -175,69 +117,75 @@ let month_reminders timestamp =
                   Unix.tm_yday  = 0;
                   Unix.tm_isdst = false
                } in
-               if min_s = "*" then
-                  let (f_rem_ts, _) = Unix.mktime temp in
-                  build_lists ((f_rem_ts, msg, filename, line_num_s) :: untimed)
-               else begin
-                  let temp_with_min = {temp with Unix.tm_min = int_of_string min_s} in
-                  let (f_rem_ts, _) = Unix.mktime temp_with_min in
-                  let duration =
-                     if duration_s = "*" then 0.0
-                     else float_of_string duration_s
-                  in
-                  (* compute the indentation level *)
-                  (* top_index and bottom_index provide the range of row indices into
-                   * array indentations that are covered by this reminder *)
-                  let top_index = 
-                     try
-                        int_of_float ((f_rem_ts -. month_start_ts) /. 3600.0)
-                     with _ -> 0
-                  in
-                  let bottom_index = 
-                     try
-                        let real_bottom_index =
-                           let shift = (f_rem_ts +. (duration *. 60.0) -. month_start_ts) /. 3600.0 in
-                           (* catch the edge effects when reminders end on hour boundaries *)
-                           if shift = float_of_int (int_of_float shift) then
-                              pred (int_of_float shift)
+               (* check whether this reminder is tagged 'nodisplay' *)
+               if not (Str.string_match nodisplay_regex tag 0) then begin
+                  let has_weight = not (Str.string_match noweight_regex tag 0) in
+                  if min_s = "*" then
+                     let (f_rem_ts, _) = Unix.mktime temp in
+                     build_lists ((f_rem_ts, msg, filename, line_num_s, has_weight) :: untimed)
+                  else begin
+                     let temp_with_min = {temp with Unix.tm_min = int_of_string min_s} in
+                     let (f_rem_ts, _) = Unix.mktime temp_with_min in
+                     let duration =
+                        if duration_s = "*" then 0.0
+                        else float_of_string duration_s
+                     in
+                     (* compute the indentation level *)
+                     (* top_index and bottom_index provide the range of row indices into
+                      * array indentations that are covered by this reminder *)
+                     let top_index = 
+                        try
+                           int_of_float ((f_rem_ts -. month_start_ts) /. 3600.0)
+                        with _ -> 0
+                     in
+                     let bottom_index = 
+                        try
+                           let real_bottom_index =
+                              let shift = (f_rem_ts +. (duration *. 60.0) -. month_start_ts) /. 3600.0 in
+                              (* catch the edge effects when reminders end on hour boundaries *)
+                              if shift = float_of_int (int_of_float shift) then
+                                 pred (int_of_float shift)
+                              else
+                                 int_of_float shift
+                           in
+                           (* the bottom index could flow off of this month, in which case
+                            * we truncate and hope everything works out *)
+                           if real_bottom_index > pred (Array.length indentations) then
+                              pred (Array.length indentations)
                            else
-                              int_of_float shift
-                        in
-                        (* the bottom index could flow off of this month, in which case
-                         * we truncate and hope everything works out *)
-                        if real_bottom_index > pred (Array.length indentations) then
-                           pred (Array.length indentations)
-                        else
-                           real_bottom_index
-                     with _ -> top_index
-                  in
-                  (* locate the smallest free indentation level *)
-                  let rec find_indent level =
-                     if level < Array.length indentations.(0) then begin
-                        let collision = ref false in
-                        for i = top_index to bottom_index do
-                           if indentations.(i).(level) then
-                              collision := true
-                           else
-                              ()
-                        done;
-                        if !collision then
-                           find_indent (succ level)
-                        else begin
+                              real_bottom_index
+                        with _ -> top_index
+                     in
+                     (* locate the smallest free indentation level *)
+                     let rec find_indent level =
+                        if level < Array.length indentations.(0) then begin
+                           let collision = ref false in
                            for i = top_index to bottom_index do
-                              indentations.(i).(level) <- true
+                              if indentations.(i).(level) then
+                                 collision := true
+                              else
+                                 ()
                            done;
-                           level
-                        end
-                     end else
-                        (* give up and default to maximum indentation *)
-                        pred (Array.length indentations.(0))
-                  in
-                  let indent = find_indent 0 in
-                  timed.(indent) <- (f_rem_ts, f_rem_ts +. (duration *. 60.), msg, 
-                                    filename, line_num_s) :: timed.(indent);
+                           if !collision then
+                              find_indent (succ level)
+                           else begin
+                              for i = top_index to bottom_index do
+                                 indentations.(i).(level) <- true
+                              done;
+                              level
+                           end
+                        end else
+                           (* give up and default to maximum indentation *)
+                           pred (Array.length indentations.(0))
+                     in
+                     let indent = find_indent 0 in
+                     timed.(indent) <- (f_rem_ts, f_rem_ts +. (duration *. 60.), msg, 
+                                       filename, line_num_s, has_weight) :: timed.(indent);
+                     build_lists untimed
+                  end
+               end else
+                  (* skip this reminder due to a 'nodisplay' tag *)
                   build_lists untimed
-               end
             end else
                (* if there was no rem_regex match, continue with next line *)
                build_lists untimed
@@ -261,62 +209,99 @@ let month_reminders timestamp =
 
 (* generate a count of how many reminders fall on any given day of
  * the month *)
-let count_reminders timed untimed =
+let count_reminders month_start_tm timed untimed =
    let rem_counts = Array.make 31 0 in
-   let count_timed (start, _, _, _, _) =
+   let count_rems start has_weight =
       let tm = Unix.localtime start in
-      let day = pred tm.Unix.tm_mday in
-      rem_counts.(day) <- succ rem_counts.(day)
+      if has_weight && tm.Unix.tm_year = month_start_tm.Unix.tm_year &&
+      tm.Unix.tm_mon = month_start_tm.Unix.tm_mon then
+         let day = pred tm.Unix.tm_mday in
+         rem_counts.(day) <- succ rem_counts.(day)
+      else
+         ()
    in
-   let count_untimed (start, _, _, _) =
-      let tm = Unix.localtime start in
-      let day = pred tm.Unix.tm_mday in
-      rem_counts.(day) <- succ rem_counts.(day)
+   let count_timed (start, _, _, _, _, has_weight) =
+      count_rems start has_weight
+   in
+   let count_untimed (start, _, _, _, has_weight) =
+      count_rems start has_weight
    in
    Array.iter (List.iter count_timed) timed;
    List.iter count_untimed untimed;
    rem_counts
 
 
-(* use cal(1) to create a calendar record for the desired timestamp *)
-let make_cal timestamp =
-   let tm = Unix.localtime timestamp in
-   let month_s = string_of_int (succ tm.Unix.tm_mon)
-   and year_s  = string_of_int (tm.Unix.tm_year + 1900) in
-   let command = 
-      (* support Euro calendar style *)
-      if !Rcfile.week_starts_monday then
-         "cal -m " ^ month_s ^ " " ^ year_s
-      else
-         "cal " ^ month_s ^ " " ^ year_s
+(* generate a count of how many hours of reminders one has on
+ * any given day of the month.  Note: some minor errors can
+ * occur during DST, but this is too small to worry about. *)
+let count_busy_hours month_start_tm timed untimed =
+   let hour_counts = Array.make 31 0.0 in
+   let last_day = 
+      let temp = {month_start_tm with Unix.tm_mday = 32} in
+      let (_, nextmonth) = Unix.mktime temp in
+      32 - nextmonth.Unix.tm_mday
    in
-   let cal_channel = Unix.open_process_in command in
-   let rec add_lines day_lines =
-      try
-         let line = input_line cal_channel in
-         add_lines (line :: day_lines)
-      with End_of_file ->
-         let _ = Unix.close_process_in cal_channel in
-         List.rev day_lines
-   in 
-   let t  = input_line cal_channel in
-   let wd = input_line cal_channel in
-   let d  = add_lines [] in {
-      title    = t;
-      weekdays = wd;
-      days     = d
-   }
+   let count_hours start stop has_weight =
+      if has_weight then
+         for day = 1 to last_day do
+            let day_tm = {month_start_tm with Unix.tm_mday = day} in
+            let (day_ts, _) = Unix.mktime day_tm in
+            if day_ts >= start then
+               if stop > day_ts +. 86400. then
+                  hour_counts.(pred day) <- hour_counts.(pred day) +. 24.
+               else if stop > day_ts then
+                  hour_counts.(pred day) <- hour_counts.(pred day) +.
+                  ((stop -. day_ts) /. 3600.)
+               else
+                  ()
+            else if day_ts +. 86400. > start then
+               if stop > day_ts +. 86400. then
+                  hour_counts.(pred day) <- hour_counts.(pred day) +.
+                  24. -. ((start -. day_ts) /. 3600.)
+               else
+                  hour_counts.(pred day) <- hour_counts.(pred day) +.
+                  ((stop -. start) /. 3600.)
+            else
+               ()
+         done
+      else
+         ()
+   in
+   let count_timed (start, stop, _, _, _, has_weight) =
+      count_hours start stop has_weight
+   in
+   let count_untimed (start, _, _, _, has_weight) =
+      let stop = start +. (!Rcfile.untimed_duration *. 60.) in
+      count_hours start stop has_weight
+   in
+   Array.iter (List.iter count_timed) timed;
+   List.iter count_untimed untimed;
+   Array.map int_of_float hour_counts
+
+
+(* determine the busy-ness level for each day in the month *)
+let count_busy month_tm timed untimed =
+   let month_start_tm = {
+      month_tm with Unix.tm_sec  = 0;
+                    Unix.tm_min  = 0;
+                    Unix.tm_hour = 0;
+                    Unix.tm_mday = 1
+   } in
+   match !Rcfile.busy_algorithm with
+   |1 -> count_reminders month_start_tm timed untimed
+   |2 -> count_busy_hours month_start_tm timed untimed
+   |_ -> Rcfile.config_failwith "busy_algorithm must be either 1 or 2"
 
 
 (* comparison functions for sorting reminders chronologically *)
 let cmp_timed rem_a rem_b =
-   let (ts_a, _, _, _, _) = rem_a
-   and (ts_b, _, _, _, _) = rem_b in
+   let (ts_a, _, _, _, _, _) = rem_a
+   and (ts_b, _, _, _, _, _) = rem_b in
    compare ts_a ts_b
 
 let cmp_untimed rem_a rem_b =
-   let (ts_a, _, _, _) = rem_a
-   and (ts_b, _, _, _) = rem_b in
+   let (ts_a, _, _, _, _) = rem_a
+   and (ts_b, _, _, _, _) = rem_b in
    compare ts_a ts_b
 
 
@@ -336,42 +321,40 @@ let safe_append a b = List.rev_append (List.rev a) b
 
 (* initialize a new three-month reminder record *)
 let create_three_month timestamp =
-   let temp = {
+   let month_start_tm = {
       Unix.localtime timestamp with Unix.tm_sec  = 0;
                                     Unix.tm_min  = 0;
                                     Unix.tm_hour = 0;
                                     Unix.tm_mday = 1
    } in
-   let (curr_ts, _) = Unix.mktime temp in
+   let (curr_ts, _) = Unix.mktime month_start_tm in
    let temp_prev = {
-      temp with Unix.tm_mon = pred temp.Unix.tm_mon
+      month_start_tm with Unix.tm_mon = pred month_start_tm.Unix.tm_mon
    } in
    let temp_next = {
-      temp with Unix.tm_mon = succ temp.Unix.tm_mon
+      month_start_tm with Unix.tm_mon = succ month_start_tm.Unix.tm_mon
    } in
    let (prev_ts, _) = Unix.mktime temp_prev
    and (next_ts, _) = Unix.mktime temp_next in
    let (pt, pu) = month_reminders prev_ts in
    let (ct, cu) = month_reminders curr_ts in
-   let (nt, nu) = month_reminders next_ts in {
+   let (nt, nu) = month_reminders next_ts in 
+   let at = Array.make (Array.length pt) [] in
+   for i = 0 to pred (Array.length at) do
+      at.(i) <- safe_append pt.(i) (safe_append ct.(i) nt.(i))
+   done;
+   let au = safe_append cu (safe_append pu nu) in {
       curr_timestamp = curr_ts;
       prev_timed     = pt;
       curr_timed     = ct;
       next_timed     = nt;
-      all_timed      = 
-         begin
-            let at = Array.make (Array.length pt) [] in
-            for i = 0 to pred (Array.length at) do
-               at.(i) <- safe_append pt.(i) (safe_append ct.(i) nt.(i))
-            done;
-            at
-         end;
+      all_timed      = at;
       prev_untimed   = pu;
       curr_untimed   = cu;
       next_untimed   = nu;
-      all_untimed    = safe_append cu (safe_append pu nu);
-      curr_counts    = count_reminders ct cu;
-      curr_cal       = make_cal curr_ts
+      all_untimed    = au;
+      curr_counts    = count_busy month_start_tm at au;
+      curr_cal       = Cal.make curr_ts !Rcfile.week_starts_monday
    }
 
 
@@ -385,28 +368,26 @@ let next_month reminders =
    let temp2 = {
       curr_tm with Unix.tm_mon = curr_tm.Unix.tm_mon + 2
    } in
-   let (new_curr_timestamp, _) = Unix.mktime temp1 in
-   let (next_timestamp, _)     = Unix.mktime temp2 in
-   let (t, u) = month_reminders next_timestamp in {
+   let (new_curr_timestamp, temp1) = Unix.mktime temp1 in
+   let (next_timestamp, temp2)     = Unix.mktime temp2 in
+   let (t, u) = month_reminders next_timestamp in 
+   let at = Array.make (Array.length t) [] in
+   for i = 0 to pred (Array.length t) do
+      at.(i) <- safe_append reminders.curr_timed.(i) 
+                (safe_append reminders.next_timed.(i) t.(i))
+   done;
+   let au = safe_append reminders.curr_untimed (safe_append reminders.next_untimed u) in {
       curr_timestamp = new_curr_timestamp;
       prev_timed     = reminders.curr_timed;
       curr_timed     = reminders.next_timed;
       next_timed     = t;
-      all_timed      =
-         begin
-            let at = Array.make (Array.length t) [] in
-            for i = 0 to pred (Array.length t) do
-               at.(i) <- safe_append reminders.curr_timed.(i) 
-                         (safe_append reminders.next_timed.(i) t.(i))
-            done;
-            at
-         end;
+      all_timed      = at;
       prev_untimed   = reminders.curr_untimed;
       curr_untimed   = reminders.next_untimed;
       next_untimed   = u;
-      all_untimed    = safe_append reminders.curr_untimed (safe_append reminders.next_untimed u);
-      curr_counts    = count_reminders reminders.next_timed reminders.next_untimed;
-      curr_cal       = make_cal new_curr_timestamp
+      all_untimed    = au;
+      curr_counts    = count_busy temp1 at au;
+      curr_cal       = Cal.make new_curr_timestamp !Rcfile.week_starts_monday
    }
 
 
@@ -419,28 +400,26 @@ let prev_month reminders =
    let temp2 = {
       curr_tm with Unix.tm_mon = curr_tm.Unix.tm_mon - 2
    } in
-   let (new_curr_timestamp, _) = Unix.mktime temp1 in
-   let (prev_timestamp, _)     = Unix.mktime temp2 in
-   let (t, u) = month_reminders prev_timestamp in {
+   let (new_curr_timestamp, temp1) = Unix.mktime temp1 in
+   let (prev_timestamp, temp2)     = Unix.mktime temp2 in
+   let (t, u) = month_reminders prev_timestamp in
+   let at = Array.make (Array.length t) [] in
+   for i = 0 to pred (Array.length t) do
+      at.(i) <- safe_append t.(i) 
+                (safe_append reminders.prev_timed.(i) reminders.curr_timed.(i))
+   done;
+   let au = safe_append u (safe_append reminders.prev_untimed reminders.curr_untimed) in {
       curr_timestamp = new_curr_timestamp;
       prev_timed     = t;
       curr_timed     = reminders.prev_timed;
       next_timed     = reminders.curr_timed;
-      all_timed      = 
-         begin
-            let at = Array.make (Array.length t) [] in
-            for i = 0 to pred (Array.length t) do
-               at.(i) <- safe_append t.(i) 
-                         (safe_append reminders.prev_timed.(i) reminders.curr_timed.(i))
-            done;
-            at
-         end;
+      all_timed      = at;
       prev_untimed   = u;
       curr_untimed   = reminders.prev_untimed;
       next_untimed   = reminders.curr_untimed;
-      all_untimed    = safe_append u (safe_append reminders.prev_untimed reminders.curr_untimed);
-      curr_counts    = count_reminders reminders.prev_timed reminders.prev_untimed;
-      curr_cal       = make_cal new_curr_timestamp
+      all_untimed    = au;
+      curr_counts    = count_busy temp1 at au;
+      curr_cal       = Cal.make new_curr_timestamp !Rcfile.week_starts_monday
    }
 
 
@@ -471,71 +450,50 @@ let update_reminders rem timestamp =
          create_three_month timestamp
 
 
+
 (* Look at all 'next' reminders after the given timestamp.  Search
  * through the list for the first occurrence of the search regexp, and return
  * a timestamp for that date.
  * This calls 'remind -n' twice--once for the current day, once for the next day.
  * The second call is necessary because reminders falling on the current day
  * but before the current timestamp will effectively suppress later recurrences
- * of that reminder. *)
+ * of that reminder. 
+ *
+ * We also have to make a separate check that the matched reminder is not tagged
+ * with 'nodisplay'; since these reminders don't show up on the display, Wyrd
+ * should not be able to match them. *)
 let find_next msg_regex timestamp =
-   let rem_regex_timed = 
-      Str.regexp "^\\([^ ]+\\)/\\([^ ]+\\)/\\([^ ]+\\) \\([0-9]+\\):\\([0-9]+\\) \\([^ ]+.*\\)$"
+   let rem_regex = 
+      Str.regexp "^\\([^ ]+\\)/\\([^ ]+\\)/\\([^ ]+\\) [^ ]+ \\([^ ]+\\) [^ ]+ \\([^ ]+\\) \\([^ ]+.*\\)$"
    in
-   let rem_regex_untimed = Str.regexp "^\\([^ ]+\\)/\\([^ ]+\\)/\\([^ ]+\\) \\([^ ]+.*\\)$" in
+   let nodisplay_regex = Str.regexp_case_fold ".*nodisplay" in
    let tm1 = Unix.localtime timestamp in
-   let tm2 = Unix.localtime (timestamp +. 86400.) in      (* 24 hours *)
+   let temp = {tm1 with Unix.tm_mday = succ tm1.Unix.tm_mday} in     (* add 24 hours *)
+   let (_, tm2) = Unix.mktime temp in
    let rem_date_str1 = (string_of_tm_mon tm1.Unix.tm_mon) ^ " " ^ 
                        (string_of_int tm1.Unix.tm_mday) ^ " " ^
                        (string_of_int (tm1.Unix.tm_year + 1900)) in
    let rem_date_str2 = (string_of_tm_mon tm2.Unix.tm_mon) ^ " " ^ 
                        (string_of_int tm2.Unix.tm_mday) ^ " " ^
                        (string_of_int (tm2.Unix.tm_year + 1900)) in
-   let remind_channel = Unix.open_process_in ("remind -n -b1 " ^ !Rcfile.reminders_file ^
-   " " ^ rem_date_str1 ^ " > /tmp/wyrd-tmp && remind -n -b1 " ^ !Rcfile.reminders_file ^
-   " " ^ rem_date_str2 ^ " | cat /tmp/wyrd-tmp - | sort") in
+   let remind_channel = Unix.open_process_in (!Rcfile.remind_command ^ " -n -s -b1 " ^ 
+   !Rcfile.reminders_file ^ " " ^ rem_date_str1 ^ " > " ^ Rcfile.tmpfile ^ " && " ^
+   !Rcfile.remind_command ^ " -n -s -b1 " ^ !Rcfile.reminders_file ^ " " ^ rem_date_str2 ^ " | cat " ^ 
+   Rcfile.tmpfile ^ " - | sort") in
    let rec check_messages () =
       try
          let line = input_line remind_channel in
-         if Str.string_match rem_regex_timed line 0 then begin
+         if Str.string_match rem_regex line 0 then begin
             (* go here if this line is a timed reminder *)
             let year  = int_of_string (Str.matched_group 1 line)
             and month = int_of_string (Str.matched_group 2 line)
             and day   = int_of_string (Str.matched_group 3 line)
-            and hour  = int_of_string (Str.matched_group 4 line)
-            and min   = int_of_string (Str.matched_group 5 line)
+            and tag   = Str.matched_group 4 line
+            and min_s = Str.matched_group 5 line
             and msg   = Str.matched_group 6 line in
             let temp = {
                Unix.tm_sec   = 0;
-               Unix.tm_min   = min;
-               Unix.tm_hour  = hour;
-               Unix.tm_mday  = day;
-               Unix.tm_mon   = pred month;
-               Unix.tm_year  = year - 1900;
-               Unix.tm_wday  = 0;
-               Unix.tm_yday  = 0;
-               Unix.tm_isdst = false
-            } in
-            let (ts, _) = Unix.mktime temp in
-            if ts > timestamp then
-               begin try
-                  let _ = Str.search_forward msg_regex msg 0 in
-                  let _ = Unix.close_process_in remind_channel in
-                  ts
-               with Not_found ->
-                  check_messages ()
-               end
-            else
-               check_messages ()
-         end else if Str.string_match rem_regex_untimed line 0 then begin
-            (* go here if this line is an untimed reminder *)
-            let year  = int_of_string (Str.matched_group 1 line)
-            and month = int_of_string (Str.matched_group 2 line)
-            and day   = int_of_string (Str.matched_group 3 line)
-            and msg   = Str.matched_group 4 line in
-            let temp = {
-               Unix.tm_sec   = 0;
-               Unix.tm_min   = 0;
+               Unix.tm_min   = if min_s = "*" then 0 else (int_of_string min_s);
                Unix.tm_hour  = 0;
                Unix.tm_mday  = day;
                Unix.tm_mon   = pred month;
@@ -548,20 +506,25 @@ let find_next msg_regex timestamp =
             if ts > timestamp then
                begin try
                   let _ = Str.search_forward msg_regex msg 0 in
-                  let _ = Unix.close_process_in remind_channel in
-                  ts
+                  (* only return the match if this value is not tagged 'nodisplay' *)
+                  if not (Str.string_match nodisplay_regex tag 0) then
+                     let _ = Unix.close_process_in remind_channel in
+                     ts
+                  else
+                     check_messages ()
                with Not_found ->
                   check_messages ()
                end
-            else
+            else begin
                check_messages ()
+            end
          end else
             (* if there was no regexp match, continue with next line *)
             check_messages ()
       with
       | End_of_file ->
          let _ = Unix.close_process_in remind_channel in
-         raise Not_found
+         raise Occurrence_not_found
       | Failure s ->
         (* if there's an error in string coersion, just drop that reminder and go
          * to the next line *)
